Ulimits - User limits utility
=============================

Ulimits is a program to allow an user to enforce the resource limits set
forth by the administrator of a GNU/Linux system. Its main purpose is to
overcome the absence of PAM on some distributions.


Purpose and rationale
---------------------
On distributions that provide [Linux-PAM](http://www.linux-pam.org/)
(that is, most distributions), resource limits are usually set in a
`/etc/security/limits.conf` file, and a [PAM module](http://www.linux-pam.org/Linux-PAM-html/sag-pam_limits.html) makes sure these limits are applied each time an user logs in. On such systems, _ulimits_ has probably no use; _ulimits_ should only be needed on systems without PAM, such as [Slackware](http://www.slackware.com/).

On Slackware, resource limits can be specified in the `/etc/limits`
file, as described in _limits(5)_. However, it seems that the only
program that cares about that file is `/bin/login`. In particular,
graphical login managers (_xdm_, _kdm_, _gdm_) ignore the `/etc/limits`
file. It means that resource limits defined in that file are only set
when the user logs in through a text console.

_ulimits_ is a small program to be installed as setuid root. It reads
the contents of the `/etc/limits` file, gets the resource limits for the
calling user, and applies them to the calling process.


Usage
-----
The intended usage of _ulimits_ is to be called from the user’s
`.xsession` script, before exec’ing the window manager:

    #!/bin/sh
    
    # Normal contents of a .xsession script
    # ...

    # Apply resource limits defined in /etc/limits
    ulimits

    # Start the window manager
    exec /usr/bin/awesome

Limits will be set for the process that calls _ulimits_, that is, the
shell running the `.xsession` script; the window manager will then
inherit the limits through the _exec_ call, and they will be propagated
to all programs launched from the window manager.

Alternatively, _ulimits_ may be called with non-option arguments; in
that case, it will set the resource limits for _its own process_, then
try to execute the program specified as the first argument. It means the
last two lines of the above script could be replaced by a single one:

    # Apply resource limits and start the window manager
    ulimits /usr/bin/awesome

This “exec mode” is the only available mode on systems which do not
provide the _prlimit(2)_ system call (non-Linux systems or Linux prior
to 2.6.36).


About the --enable-merged-limits option
---------------------------------------
The default behavior of _ulimits_ when interpreting the `/etc/limits`
file mimics the behavor of `/bin/login`, in the sense that only one line
is ever considered, even if several lines could apply to the calling
user.

The configure-time option `--enable-merged-limits`, introduced in
version 0.3.2, allows to change that behavior. If enabled, all the lines
that could apply to the calling user will be treated as if they were a
single line.

(Since this is not the standard interpretation of `/etc/limits`,
administrators should review carefully the contents of that file on
their system before enabling this feature, to make sure they are not
inadvertently giving an user more privileges than they originally
intended.)


About the program running as setuid root
----------------------------------------
_ulimits_ must run as setuid root for two reasons. First, the
_limits(5)_ page says that the `/etc/limits` file should be readable
only by root. Second, only a privileged process can set the resource
limits to arbitrary values; an unprivileged process can only set them to
lower values than the default limits.

When called in “exec mode”, root privileges are dropped once the limits
are set and before exec’ing the specified program.


Copying
-------
Ulimits is distributed under the terms of the GNU General Public
License, version 3 or higher. The full license is included in the
[COPYING file](COPYING) of the source distribution.


Homepage and repository
-----------------------
The project homepage, where release tarballs may be found, is located at
https://incenp.org/dvlpt/ulimits.html. The latest source code is
available at git://git.incenp.org/ulimits.git and may also be browsed at
https://incenp.org/dvlpt/gitweb/?p=ulimits.git;a=summary.
